"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .billingcadence import BillingCadence
from .billingmodel import BillingModel
from .billingperiod import BillingPeriod
from .billingtier import BillingTier
from .invoicecadence import InvoiceCadence
from .price_jsonbtransformquantity import (
    PriceJSONBTransformQuantity,
    PriceJSONBTransformQuantityTypedDict,
)
from .price_pricetier import PricePriceTier, PricePriceTierTypedDict
from .priceentitytype import PriceEntityType
from .pricetype import PriceType
from .priceunittype import PriceUnitType
from .status import Status
from flexprice_py.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from pydantic import model_serializer
from typing import Dict, List, Optional
from typing_extensions import NotRequired, TypedDict


class PricePriceTypedDict(TypedDict):
    amount: NotRequired[str]
    r"""Amount stored in main currency units (e.g., dollars, not cents)
    For USD: 12.50 means $12.50
    """
    billing_cadence: NotRequired[BillingCadence]
    billing_model: NotRequired[BillingModel]
    billing_period: NotRequired[BillingPeriod]
    billing_period_count: NotRequired[int]
    r"""BillingPeriodCount is the count of the billing period ex 1, 3, 6, 12"""
    conversion_rate: NotRequired[str]
    r"""ConversionRate is the conversion rate of the price unit to the fiat currency"""
    created_at: NotRequired[str]
    created_by: NotRequired[str]
    currency: NotRequired[str]
    r"""Currency 3 digit ISO currency code in lowercase ex usd, eur, gbp"""
    description: NotRequired[str]
    r"""Description of the price"""
    display_amount: NotRequired[str]
    r"""DisplayAmount is the formatted amount with currency symbol
    For USD: $12.50
    """
    display_name: NotRequired[str]
    r"""DisplayName is the name of the price"""
    display_price_unit_amount: NotRequired[str]
    r"""DisplayPriceUnitAmount is the formatted amount of the price unit"""
    end_date: NotRequired[str]
    r"""EndDate is the end date of the price"""
    entity_id: NotRequired[str]
    r"""EntityID holds the value of the \"entity_id\" field."""
    entity_type: NotRequired[PriceEntityType]
    environment_id: NotRequired[str]
    r"""EnvironmentID is the environment identifier for the price"""
    group_id: NotRequired[str]
    r"""GroupID references the group this price belongs to"""
    id: NotRequired[str]
    r"""ID uuid identifier for the price"""
    invoice_cadence: NotRequired[InvoiceCadence]
    lookup_key: NotRequired[str]
    r"""LookupKey used for looking up the price in the database"""
    metadata: NotRequired[Dict[str, str]]
    meter_id: NotRequired[str]
    r"""MeterID is the id of the meter for usage based pricing"""
    min_quantity: NotRequired[Nullable[str]]
    r"""MinQuantity is the minimum quantity of the price"""
    parent_price_id: NotRequired[str]
    r"""ParentPriceID references the root price (always set for price lineage tracking)"""
    price_unit: NotRequired[str]
    r"""PriceUnit is the code of the price unit (e.g., 'btc', 'eth')"""
    price_unit_amount: NotRequired[str]
    r"""PriceUnitAmount is the amount of the price unit"""
    price_unit_id: NotRequired[str]
    r"""PriceUnitID is the id of the price unit (for CUSTOM type)"""
    price_unit_tiers: NotRequired[List[PricePriceTierTypedDict]]
    r"""PriceUnitTiers are the tiers for the price unit when BillingModel is TIERED"""
    price_unit_type: NotRequired[PriceUnitType]
    start_date: NotRequired[str]
    r"""StartDate is the start date of the price"""
    status: NotRequired[Status]
    tenant_id: NotRequired[str]
    tier_mode: NotRequired[BillingTier]
    tiers: NotRequired[List[PricePriceTierTypedDict]]
    transform_quantity: NotRequired[PriceJSONBTransformQuantityTypedDict]
    trial_period: NotRequired[int]
    r"""TrialPeriod is the number of days for the trial period
    Note: This is only applicable for recurring prices (BILLING_CADENCE_RECURRING)
    """
    type: NotRequired[PriceType]
    updated_at: NotRequired[str]
    updated_by: NotRequired[str]


class PricePrice(BaseModel):
    amount: Optional[str] = None
    r"""Amount stored in main currency units (e.g., dollars, not cents)
    For USD: 12.50 means $12.50
    """

    billing_cadence: Optional[BillingCadence] = None

    billing_model: Optional[BillingModel] = None

    billing_period: Optional[BillingPeriod] = None

    billing_period_count: Optional[int] = None
    r"""BillingPeriodCount is the count of the billing period ex 1, 3, 6, 12"""

    conversion_rate: Optional[str] = None
    r"""ConversionRate is the conversion rate of the price unit to the fiat currency"""

    created_at: Optional[str] = None

    created_by: Optional[str] = None

    currency: Optional[str] = None
    r"""Currency 3 digit ISO currency code in lowercase ex usd, eur, gbp"""

    description: Optional[str] = None
    r"""Description of the price"""

    display_amount: Optional[str] = None
    r"""DisplayAmount is the formatted amount with currency symbol
    For USD: $12.50
    """

    display_name: Optional[str] = None
    r"""DisplayName is the name of the price"""

    display_price_unit_amount: Optional[str] = None
    r"""DisplayPriceUnitAmount is the formatted amount of the price unit"""

    end_date: Optional[str] = None
    r"""EndDate is the end date of the price"""

    entity_id: Optional[str] = None
    r"""EntityID holds the value of the \"entity_id\" field."""

    entity_type: Optional[PriceEntityType] = None

    environment_id: Optional[str] = None
    r"""EnvironmentID is the environment identifier for the price"""

    group_id: Optional[str] = None
    r"""GroupID references the group this price belongs to"""

    id: Optional[str] = None
    r"""ID uuid identifier for the price"""

    invoice_cadence: Optional[InvoiceCadence] = None

    lookup_key: Optional[str] = None
    r"""LookupKey used for looking up the price in the database"""

    metadata: Optional[Dict[str, str]] = None

    meter_id: Optional[str] = None
    r"""MeterID is the id of the meter for usage based pricing"""

    min_quantity: OptionalNullable[str] = UNSET
    r"""MinQuantity is the minimum quantity of the price"""

    parent_price_id: Optional[str] = None
    r"""ParentPriceID references the root price (always set for price lineage tracking)"""

    price_unit: Optional[str] = None
    r"""PriceUnit is the code of the price unit (e.g., 'btc', 'eth')"""

    price_unit_amount: Optional[str] = None
    r"""PriceUnitAmount is the amount of the price unit"""

    price_unit_id: Optional[str] = None
    r"""PriceUnitID is the id of the price unit (for CUSTOM type)"""

    price_unit_tiers: Optional[List[PricePriceTier]] = None
    r"""PriceUnitTiers are the tiers for the price unit when BillingModel is TIERED"""

    price_unit_type: Optional[PriceUnitType] = None

    start_date: Optional[str] = None
    r"""StartDate is the start date of the price"""

    status: Optional[Status] = None

    tenant_id: Optional[str] = None

    tier_mode: Optional[BillingTier] = None

    tiers: Optional[List[PricePriceTier]] = None

    transform_quantity: Optional[PriceJSONBTransformQuantity] = None

    trial_period: Optional[int] = None
    r"""TrialPeriod is the number of days for the trial period
    Note: This is only applicable for recurring prices (BILLING_CADENCE_RECURRING)
    """

    type: Optional[PriceType] = None

    updated_at: Optional[str] = None

    updated_by: Optional[str] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "amount",
                "billing_cadence",
                "billing_model",
                "billing_period",
                "billing_period_count",
                "conversion_rate",
                "created_at",
                "created_by",
                "currency",
                "description",
                "display_amount",
                "display_name",
                "display_price_unit_amount",
                "end_date",
                "entity_id",
                "entity_type",
                "environment_id",
                "group_id",
                "id",
                "invoice_cadence",
                "lookup_key",
                "metadata",
                "meter_id",
                "min_quantity",
                "parent_price_id",
                "price_unit",
                "price_unit_amount",
                "price_unit_id",
                "price_unit_tiers",
                "price_unit_type",
                "start_date",
                "status",
                "tenant_id",
                "tier_mode",
                "tiers",
                "transform_quantity",
                "trial_period",
                "type",
                "updated_at",
                "updated_by",
            ]
        )
        nullable_fields = set(["min_quantity"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            is_nullable_and_explicitly_set = (
                k in nullable_fields
                and (self.__pydantic_fields_set__.intersection({n}))  # pylint: disable=no-member
            )

            if val != UNSET_SENTINEL:
                if (
                    val is not None
                    or k not in optional_fields
                    or is_nullable_and_explicitly_set
                ):
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from flexprice_py.models.components import (
    dto_createintegrationrequest as components_dto_createintegrationrequest,
)
from flexprice_py.types import BaseModel
from flexprice_py.utils import FieldMetadata, PathParamMetadata, RequestMetadata
import pydantic
from typing_extensions import Annotated, TypedDict


class CreateOrUpdateIntegrationRequestTypedDict(TypedDict):
    provider_param: str
    r"""Integration provider"""
    body: components_dto_createintegrationrequest.DtoCreateIntegrationRequestTypedDict
    r"""Integration creation request"""


class CreateOrUpdateIntegrationRequest(BaseModel):
    provider_param: Annotated[
        str,
        pydantic.Field(alias="provider"),
        FieldMetadata(path=PathParamMetadata(style="simple", explode=False)),
    ]
    r"""Integration provider"""

    body: Annotated[
        components_dto_createintegrationrequest.DtoCreateIntegrationRequest,
        FieldMetadata(request=RequestMetadata(media_type="application/json")),
    ]
    r"""Integration creation request"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from flexprice_py.types import BaseModel, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class ModelsTemporalWorkflowResultTypedDict(TypedDict):
    message: NotRequired[str]
    run_id: NotRequired[str]
    workflow_id: NotRequired[str]


class ModelsTemporalWorkflowResult(BaseModel):
    message: Optional[str] = None

    run_id: Optional[str] = None

    workflow_id: Optional[str] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["message", "run_id", "workflow_id"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from flexprice_py.types import BaseModel, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class DtoUsageAnalyticPointTypedDict(TypedDict):
    computed_commitment_utilized_amount: NotRequired[str]
    r"""Commitment breakdown (only populated for windowed commitments)"""
    computed_overage_amount: NotRequired[str]
    computed_true_up_amount: NotRequired[str]
    cost: NotRequired[str]
    event_count: NotRequired[int]
    r"""Number of events in this time window"""
    timestamp: NotRequired[str]
    usage: NotRequired[str]


class DtoUsageAnalyticPoint(BaseModel):
    computed_commitment_utilized_amount: Optional[str] = None
    r"""Commitment breakdown (only populated for windowed commitments)"""

    computed_overage_amount: Optional[str] = None

    computed_true_up_amount: Optional[str] = None

    cost: Optional[str] = None

    event_count: Optional[int] = None
    r"""Number of events in this time window"""

    timestamp: Optional[str] = None

    usage: Optional[str] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "computed_commitment_utilized_amount",
                "computed_overage_amount",
                "computed_true_up_amount",
                "cost",
                "event_count",
                "timestamp",
                "usage",
            ]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from flexprice_py.types import BaseModel
from typing_extensions import TypedDict


class ScheduleUpdateBillingPeriodRequestTypedDict(TypedDict):
    r"""Schedule Update Billing Period Request"""


class ScheduleUpdateBillingPeriodRequest(BaseModel):
    r"""Schedule Update Billing Period Request"""


class ScheduleUpdateBillingPeriodResponseTypedDict(TypedDict):
    r"""OK"""


class ScheduleUpdateBillingPeriodResponse(BaseModel):
    r"""OK"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from flexprice_py.types import BaseModel, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Dict, Optional
from typing_extensions import NotRequired, TypedDict


class DtoUsageBreakdownItemTypedDict(TypedDict):
    cost: NotRequired[str]
    r"""cost is the cost attributed to this group for the line item"""
    event_count: NotRequired[int]
    r"""event_count is the number of events from this group (optional)"""
    grouped_by: NotRequired[Dict[str, str]]
    r"""grouped_by contains the grouping field values (e.g., {\"source\": \"api\", \"org_id\": \"org123\"})"""
    percentage: NotRequired[str]
    r"""percentage is the percentage of total line item cost from this group (optional)"""
    usage: NotRequired[str]
    r"""usage is the total usage amount from this group (optional, for additional context)"""


class DtoUsageBreakdownItem(BaseModel):
    cost: Optional[str] = None
    r"""cost is the cost attributed to this group for the line item"""

    event_count: Optional[int] = None
    r"""event_count is the number of events from this group (optional)"""

    grouped_by: Optional[Dict[str, str]] = None
    r"""grouped_by contains the grouping field values (e.g., {\"source\": \"api\", \"org_id\": \"org123\"})"""

    percentage: Optional[str] = None
    r"""percentage is the percentage of total line item cost from this group (optional)"""

    usage: Optional[str] = None
    r"""usage is the total usage amount from this group (optional, for additional context)"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            ["cost", "event_count", "grouped_by", "percentage", "usage"]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .secretprovider import SecretProvider
from flexprice_py.types import BaseModel
from typing import Dict
from typing_extensions import TypedDict


class DtoCreateIntegrationRequestTypedDict(TypedDict):
    credentials: Dict[str, str]
    name: str
    provider: SecretProvider


class DtoCreateIntegrationRequest(BaseModel):
    credentials: Dict[str, str]

    name: str

    provider: SecretProvider



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .basesdk import BaseSDK
from flexprice_py import errors, models, utils
from flexprice_py._hooks import HookContext
from flexprice_py.types import OptionalNullable, UNSET
from flexprice_py.utils.unmarshal_json_response import unmarshal_json_response
from typing import Any, Dict, List, Mapping, Optional, Union


class Prices(BaseSDK):
    def create_price(
        self,
        *,
        billing_cadence: models.components.BillingCadence,
        billing_model: models.components.BillingModel,
        billing_period: models.components.BillingPeriod,
        currency: str,
        entity_id: str,
        entity_type: models.components.PriceEntityType,
        invoice_cadence: models.components.InvoiceCadence,
        price_unit_type: models.components.PriceUnitType,
        type_: models.components.PriceType,
        amount: Optional[str] = None,
        billing_period_count: Optional[int] = None,
        description: Optional[str] = None,
        display_name: Optional[str] = None,
        end_date: Optional[str] = None,
        filter_values: Optional[Dict[str, List[str]]] = None,
        group_id: Optional[str] = None,
        lookup_key: Optional[str] = None,
        metadata: Optional[Dict[str, str]] = None,
        meter_id: Optional[str] = None,
        min_quantity: Optional[int] = None,
        price_unit_config: Optional[
            Union[
                models.components.DtoPriceUnitConfig,
                models.components.DtoPriceUnitConfigTypedDict,
            ]
        ] = None,
        start_date: Optional[str] = None,
        tier_mode: Optional[models.components.BillingTier] = None,
        tiers: Optional[
            Union[
                List[models.components.DtoCreatePriceTier],
                List[models.components.DtoCreatePriceTierTypedDict],
            ]
        ] = None,
        transform_quantity: Optional[
            Union[
                models.components.PriceTransformQuantity,
                models.components.PriceTransformQuantityTypedDict,
            ]
        ] = None,
        trial_period: Optional[int] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.components.DtoPriceResponse:
        r"""Create price

        Use when adding a new price to a plan or catalog (e.g. per-seat, flat, or metered). Ideal for both simple and usage-based pricing.

        :param billing_cadence:
        :param billing_model:
        :param billing_period:
        :param currency:
        :param entity_id:
        :param entity_type:
        :param invoice_cadence:
        :param price_unit_type:
        :param type:
        :param amount:
        :param billing_period_count:
        :param description:
        :param display_name:
        :param end_date:
        :param filter_values:
        :param group_id: GroupID is the id of the group to add the price to
        :param lookup_key:
        :param metadata:
        :param meter_id:
        :param min_quantity: MinQuantity is the minimum quantity of the price
        :param price_unit_config:
        :param start_date:
        :param tier_mode:
        :param tiers:
        :param transform_quantity:
        :param trial_period:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.components.DtoCreatePriceRequest(
            amount=amount,
            billing_cadence=billing_cadence,
            billing_model=billing_model,
            billing_period=billing_period,
            billing_period_count=billing_period_count,
            currency=currency,
            description=description,
            display_name=display_name,
            end_date=end_date,
            entity_id=entity_id,
            entity_type=entity_type,
            filter_values=filter_values,
            group_id=group_id,
            invoice_cadence=invoice_cadence,
            lookup_key=lookup_key,
            metadata=metadata,
            meter_id=meter_id,
            min_quantity=min_quantity,
            price_unit_config=utils.get_pydantic_model(
                price_unit_config, Optional[models.components.DtoPriceUnitConfig]
            ),
            price_unit_type=price_unit_type,
            start_date=start_date,
            tier_mode=tier_mode,
            tiers=utils.get_pydantic_model(
                tiers, Optional[List[models.components.DtoCreatePriceTier]]
            ),
            transform_quantity=utils.get_pydantic_model(
                transform_quantity, Optional[models.components.PriceTransformQuantity]
            ),
            trial_period=trial_period,
            type=type_,
        )

        req = self._build_request(
            method="POST",
            path="/prices",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request, False, False, "json", models.components.DtoCreatePriceRequest
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="createPrice",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "201", "application/json"):
            return unmarshal_json_response(models.components.DtoPriceResponse, http_res)
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.FlexpriceDefaultError("Unexpected response received", http_res)

    async def create_price_async(
        self,
        *,
        billing_cadence: models.components.BillingCadence,
        billing_model: models.components.BillingModel,
        billing_period: models.components.BillingPeriod,
        currency: str,
        entity_id: str,
        entity_type: models.components.PriceEntityType,
        invoice_cadence: models.components.InvoiceCadence,
        price_unit_type: models.components.PriceUnitType,
        type_: models.components.PriceType,
        amount: Optional[str] = None,
        billing_period_count: Optional[int] = None,
        description: Optional[str] = None,
        display_name: Optional[str] = None,
        end_date: Optional[str] = None,
        filter_values: Optional[Dict[str, List[str]]] = None,
        group_id: Optional[str] = None,
        lookup_key: Optional[str] = None,
        metadata: Optional[Dict[str, str]] = None,
        meter_id: Optional[str] = None,
        min_quantity: Optional[int] = None,
        price_unit_config: Optional[
            Union[
                models.components.DtoPriceUnitConfig,
                models.components.DtoPriceUnitConfigTypedDict,
            ]
        ] = None,
        start_date: Optional[str] = None,
        tier_mode: Optional[models.components.BillingTier] = None,
        tiers: Optional[
            Union[
                List[models.components.DtoCreatePriceTier],
                List[models.components.DtoCreatePriceTierTypedDict],
            ]
        ] = None,
        transform_quantity: Optional[
            Union[
                models.components.PriceTransformQuantity,
                models.components.PriceTransformQuantityTypedDict,
            ]
        ] = None,
        trial_period: Optional[int] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.components.DtoPriceResponse:
        r"""Create price

        Use when adding a new price to a plan or catalog (e.g. per-seat, flat, or metered). Ideal for both simple and usage-based pricing.

        :param billing_cadence:
        :param billing_model:
        :param billing_period:
        :param currency:
        :param entity_id:
        :param entity_type:
        :param invoice_cadence:
        :param price_unit_type:
        :param type:
        :param amount:
        :param billing_period_count:
        :param description:
        :param display_name:
        :param end_date:
        :param filter_values:
        :param group_id: GroupID is the id of the group to add the price to
        :param lookup_key:
        :param metadata:
        :param meter_id:
        :param min_quantity: MinQuantity is the minimum quantity of the price
        :param price_unit_config:
        :param start_date:
        :param tier_mode:
        :param tiers:
        :param transform_quantity:
        :param trial_period:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.components.DtoCreatePriceRequest(
            amount=amount,
            billing_cadence=billing_cadence,
            billing_model=billing_model,
            billing_period=billing_period,
            billing_period_count=billing_period_count,
            currency=currency,
            description=description,
            display_name=display_name,
            end_date=end_date,
            entity_id=entity_id,
            entity_type=entity_type,
            filter_values=filter_values,
            group_id=group_id,
            invoice_cadence=invoice_cadence,
            lookup_key=lookup_key,
            metadata=metadata,
            meter_id=meter_id,
            min_quantity=min_quantity,
            price_unit_config=utils.get_pydantic_model(
                price_unit_config, Optional[models.components.DtoPriceUnitConfig]
            ),
            price_unit_type=price_unit_type,
            start_date=start_date,
            tier_mode=tier_mode,
            tiers=utils.get_pydantic_model(
                tiers, Optional[List[models.components.DtoCreatePriceTier]]
            ),
            transform_quantity=utils.get_pydantic_model(
                transform_quantity, Optional[models.components.PriceTransformQuantity]
            ),
            trial_period=trial_period,
            type=type_,
        )

        req = self._build_request_async(
            method="POST",
            path="/prices",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request, False, False, "json", models.components.DtoCreatePriceRequest
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="createPrice",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "201", "application/json"):
            return unmarshal_json_response(models.components.DtoPriceResponse, http_res)
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.FlexpriceDefaultError("Unexpected response received", http_res)

    def create_prices_bulk(
        self,
        *,
        items: Union[
            List[models.components.DtoCreatePriceRequest],
            List[models.components.DtoCreatePriceRequestTypedDict],
        ],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.components.DtoCreateBulkPriceResponse:
        r"""Create prices in bulk

        Use when creating many prices at once (e.g. importing a catalog or setting up a plan with multiple tiers).

        :param items:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.components.DtoCreateBulkPriceRequest(
            items=utils.get_pydantic_model(
                items, List[models.components.DtoCreatePriceRequest]
            ),
        )

        req = self._build_request(
            method="POST",
            path="/prices/bulk",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request,
                False,
                False,
                "json",
                models.components.DtoCreateBulkPriceRequest,
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="createPricesBulk",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "201", "application/json"):
            return unmarshal_json_response(
                models.components.DtoCreateBulkPriceResponse, http_res
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.FlexpriceDefaultError("Unexpected response received", http_res)

    async def create_prices_bulk_async(
        self,
        *,
        items: Union[
            List[models.components.DtoCreatePriceRequest],
            List[models.components.DtoCreatePriceRequestTypedDict],
        ],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.components.DtoCreateBulkPriceResponse:
        r"""Create prices in bulk

        Use when creating many prices at once (e.g. importing a catalog or setting up a plan with multiple tiers).

        :param items:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.components.DtoCreateBulkPriceRequest(
            items=utils.get_pydantic_model(
                items, List[models.components.DtoCreatePriceRequest]
            ),
        )

        req = self._build_request_async(
            method="POST",
            path="/prices/bulk",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request,
                False,
                False,
                "json",
                models.components.DtoCreateBulkPriceRequest,
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="createPricesBulk",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "201", "application/json"):
            return unmarshal_json_response(
                models.components.DtoCreateBulkPriceResponse, http_res
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.FlexpriceDefaultError("Unexpected response received", http_res)

    def get_price_by_lookup_key(
        self,
        *,
        lookup_key: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.components.DtoPriceResponse:
        r"""Get price by lookup key

        Use when resolving a price by external id (e.g. from your catalog or CMS). Ideal for integrations.

        :param lookup_key: Lookup key
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.operations.GetPriceByLookupKeyRequest(
            lookup_key=lookup_key,
        )

        req = self._build_request(
            method="GET",
            path="/prices/lookup/{lookup_key}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="getPriceByLookupKey",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.components.DtoPriceResponse, http_res)
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.FlexpriceDefaultError("Unexpected response received", http_res)

    async def get_price_by_lookup_key_async(
        self,
        *,
        lookup_key: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.components.DtoPriceResponse:
        r"""Get price by lookup key

        Use when resolving a price by external id (e.g. from your catalog or CMS). Ideal for integrations.

        :param lookup_key: Lookup key
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.operations.GetPriceByLookupKeyRequest(
            lookup_key=lookup_key,
        )

        req = self._build_request_async(
            method="GET",
            path="/prices/lookup/{lookup_key}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="getPriceByLookupKey",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.components.DtoPriceResponse, http_res)
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.FlexpriceDefaultError("Unexpected response received", http_res)

    def query_price(
        self,
        *,
        allow_expired_prices: Optional[bool] = False,
        end_time: Optional[str] = None,
        entity_ids: Optional[List[str]] = None,
        entity_type: Optional[models.components.PriceEntityType] = None,
        expand: Optional[str] = None,
        filters: Optional[
            Union[
                List[models.components.FilterCondition],
                List[models.components.FilterConditionTypedDict],
            ]
        ] = None,
        limit: Optional[int] = None,
        meter_ids: Optional[List[str]] = None,
        offset: Optional[int] = None,
        order: Optional[models.components.PriceFilterOrder] = None,
        parent_price_id: Optional[str] = None,
        plan_ids: Optional[List[str]] = None,
        price_ids: Optional[List[str]] = None,
        sort: Optional[str] = None,
        start_date_lt: Optional[str] = None,
        start_time: Optional[str] = None,
        status: Optional[models.components.Status] = None,
        subscription_id: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.components.DtoListPricesResponse:
        r"""Query prices

        Use when listing or searching prices (e.g. plan builder or catalog). Returns a paginated list; supports filtering and sorting.

        :param allow_expired_prices:
        :param end_time:
        :param entity_ids:
        :param entity_type:
        :param expand:
        :param filters: DSL filters
        :param limit:
        :param meter_ids:
        :param offset:
        :param order:
        :param parent_price_id:
        :param plan_ids: Price override filtering fields
        :param price_ids:
        :param sort:
        :param start_date_lt:
        :param start_time:
        :param status:
        :param subscription_id:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.components.PriceFilter(
            allow_expired_prices=allow_expired_prices,
            end_time=end_time,
            entity_ids=entity_ids,
            entity_type=entity_type,
            expand=expand,
            filters=utils.get_pydantic_model(
                filters, Optional[List[models.components.FilterCondition]]
            ),
            limit=limit,
            meter_ids=meter_ids,
            offset=offset,
            order=order,
            parent_price_id=parent_price_id,
            plan_ids=plan_ids,
            price_ids=price_ids,
            sort=sort,
            start_date_lt=start_date_lt,
            start_time=start_time,
            status=status,
            subscription_id=subscription_id,
        )

        req = self._build_request(
            method="POST",
            path="/prices/search",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request, False, False, "json", models.components.PriceFilter
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="queryPrice",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.components.DtoListPricesResponse, http_res
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.FlexpriceDefaultError("Unexpected response received", http_res)

    async def query_price_async(
        self,
        *,
        allow_expired_prices: Optional[bool] = False,
        end_time: Optional[str] = None,
        entity_ids: Optional[List[str]] = None,
        entity_type: Optional[models.components.PriceEntityType] = None,
        expand: Optional[str] = None,
        filters: Optional[
            Union[
                List[models.components.FilterCondition],
                List[models.components.FilterConditionTypedDict],
            ]
        ] = None,
        limit: Optional[int] = None,
        meter_ids: Optional[List[str]] = None,
        offset: Optional[int] = None,
        order: Optional[models.components.PriceFilterOrder] = None,
        parent_price_id: Optional[str] = None,
        plan_ids: Optional[List[str]] = None,
        price_ids: Optional[List[str]] = None,
        sort: Optional[str] = None,
        start_date_lt: Optional[str] = None,
        start_time: Optional[str] = None,
        status: Optional[models.components.Status] = None,
        subscription_id: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.components.DtoListPricesResponse:
        r"""Query prices

        Use when listing or searching prices (e.g. plan builder or catalog). Returns a paginated list; supports filtering and sorting.

        :param allow_expired_prices:
        :param end_time:
        :param entity_ids:
        :param entity_type:
        :param expand:
        :param filters: DSL filters
        :param limit:
        :param meter_ids:
        :param offset:
        :param order:
        :param parent_price_id:
        :param plan_ids: Price override filtering fields
        :param price_ids:
        :param sort:
        :param start_date_lt:
        :param start_time:
        :param status:
        :param subscription_id:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.components.PriceFilter(
            allow_expired_prices=allow_expired_prices,
            end_time=end_time,
            entity_ids=entity_ids,
            entity_type=entity_type,
            expand=expand,
            filters=utils.get_pydantic_model(
                filters, Optional[List[models.components.FilterCondition]]
            ),
            limit=limit,
            meter_ids=meter_ids,
            offset=offset,
            order=order,
            parent_price_id=parent_price_id,
            plan_ids=plan_ids,
            price_ids=price_ids,
            sort=sort,
            start_date_lt=start_date_lt,
            start_time=start_time,
            status=status,
            subscription_id=subscription_id,
        )

        req = self._build_request_async(
            method="POST",
            path="/prices/search",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request, False, False, "json", models.components.PriceFilter
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="queryPrice",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.components.DtoListPricesResponse, http_res
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.FlexpriceDefaultError("Unexpected response received", http_res)

    def get_price(
        self,
        *,
        id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.components.DtoPriceResponse:
        r"""Get price

        Use when you need to load a single price (e.g. for display or editing). Response includes expanded meter and price unit when applicable.

        :param id: Price ID
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.operations.GetPriceRequest(
            id=id,
        )

        req = self._build_request(
            method="GET",
            path="/prices/{id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="getPrice",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.components.DtoPriceResponse, http_res)
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.FlexpriceDefaultError("Unexpected response received", http_res)

    async def get_price_async(
        self,
        *,
        id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.components.DtoPriceResponse:
        r"""Get price

        Use when you need to load a single price (e.g. for display or editing). Response includes expanded meter and price unit when applicable.

        :param id: Price ID
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.operations.GetPriceRequest(
            id=id,
        )

        req = self._build_request_async(
            method="GET",
            path="/prices/{id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="getPrice",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.components.DtoPriceResponse, http_res)
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.FlexpriceDefaultError("Unexpected response received", http_res)

    def update_price(
        self,
        *,
        id: str,
        amount: Optional[str] = None,
        billing_model: Optional[models.components.BillingModel] = None,
        description: Optional[str] = None,
        display_name: Optional[str] = None,
        effective_from: Optional[str] = None,
        group_id: Optional[str] = None,
        lookup_key: Optional[str] = None,
        metadata: Optional[Dict[str, str]] = None,
        price_unit_amount: Optional[str] = None,
        price_unit_tiers: Optional[
            Union[
                List[models.components.DtoCreatePriceTier],
                List[models.components.DtoCreatePriceTierTypedDict],
            ]
        ] = None,
        tier_mode: Optional[models.components.BillingTier] = None,
        tiers: Optional[
            Union[
                List[models.components.DtoCreatePriceTier],
                List[models.components.DtoCreatePriceTierTypedDict],
            ]
        ] = None,
        transform_quantity: Optional[
            Union[
                models.components.PriceTransformQuantity,
                models.components.PriceTransformQuantityTypedDict,
            ]
        ] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.components.DtoPriceResponse:
        r"""Update price

        Use when changing price configuration (e.g. amount, billing scheme, or metadata).

        :param id: Price ID
        :param amount: Amount is the new price amount that overrides the original price (optional)
        :param billing_model:
        :param description:
        :param display_name:
        :param effective_from:
        :param group_id: GroupID is the id of the group to update the price in
        :param lookup_key: All price fields that can be updated
            Non-critical fields (can be updated directly)
        :param metadata:
        :param price_unit_amount: PriceUnitAmount is the price unit amount (for CUSTOM price unit type, FLAT_FEE/PACKAGE billing models)
        :param price_unit_tiers: PriceUnitTiers are the price unit tiers (for CUSTOM price unit type, TIERED billing model)
        :param tier_mode:
        :param tiers: Tiers determines the pricing tiers for this line item
        :param transform_quantity:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.operations.UpdatePriceRequest(
            id=id,
            body=models.components.DtoUpdatePriceRequest(
                amount=amount,
                billing_model=billing_model,
                description=description,
                display_name=display_name,
                effective_from=effective_from,
                group_id=group_id,
                lookup_key=lookup_key,
                metadata=metadata,
                price_unit_amount=price_unit_amount,
                price_unit_tiers=utils.get_pydantic_model(
                    price_unit_tiers,
                    Optional[List[models.components.DtoCreatePriceTier]],
                ),
                tier_mode=tier_mode,
                tiers=utils.get_pydantic_model(
                    tiers, Optional[List[models.components.DtoCreatePriceTier]]
                ),
                transform_quantity=utils.get_pydantic_model(
                    transform_quantity,
                    Optional[models.components.PriceTransformQuantity],
                ),
            ),
        )

        req = self._build_request(
            method="PUT",
            path="/prices/{id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.body,
                False,
                False,
                "json",
                models.components.DtoUpdatePriceRequest,
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="updatePrice",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.components.DtoPriceResponse, http_res)
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.FlexpriceDefaultError("Unexpected response received", http_res)

    async def update_price_async(
        self,
        *,
        id: str,
        amount: Optional[str] = None,
        billing_model: Optional[models.components.BillingModel] = None,
        description: Optional[str] = None,
        display_name: Optional[str] = None,
        effective_from: Optional[str] = None,
        group_id: Optional[str] = None,
        lookup_key: Optional[str] = None,
        metadata: Optional[Dict[str, str]] = None,
        price_unit_amount: Optional[str] = None,
        price_unit_tiers: Optional[
            Union[
                List[models.components.DtoCreatePriceTier],
                List[models.components.DtoCreatePriceTierTypedDict],
            ]
        ] = None,
        tier_mode: Optional[models.components.BillingTier] = None,
        tiers: Optional[
            Union[
                List[models.components.DtoCreatePriceTier],
                List[models.components.DtoCreatePriceTierTypedDict],
            ]
        ] = None,
        transform_quantity: Optional[
            Union[
                models.components.PriceTransformQuantity,
                models.components.PriceTransformQuantityTypedDict,
            ]
        ] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.components.DtoPriceResponse:
        r"""Update price

        Use when changing price configuration (e.g. amount, billing scheme, or metadata).

        :param id: Price ID
        :param amount: Amount is the new price amount that overrides the original price (optional)
        :param billing_model:
        :param description:
        :param display_name:
        :param effective_from:
        :param group_id: GroupID is the id of the group to update the price in
        :param lookup_key: All price fields that can be updated
            Non-critical fields (can be updated directly)
        :param metadata:
        :param price_unit_amount: PriceUnitAmount is the price unit amount (for CUSTOM price unit type, FLAT_FEE/PACKAGE billing models)
        :param price_unit_tiers: PriceUnitTiers are the price unit tiers (for CUSTOM price unit type, TIERED billing model)
        :param tier_mode:
        :param tiers: Tiers determines the pricing tiers for this line item
        :param transform_quantity:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.operations.UpdatePriceRequest(
            id=id,
            body=models.components.DtoUpdatePriceRequest(
                amount=amount,
                billing_model=billing_model,
                description=description,
                display_name=display_name,
                effective_from=effective_from,
                group_id=group_id,
                lookup_key=lookup_key,
                metadata=metadata,
                price_unit_amount=price_unit_amount,
                price_unit_tiers=utils.get_pydantic_model(
                    price_unit_tiers,
                    Optional[List[models.components.DtoCreatePriceTier]],
                ),
                tier_mode=tier_mode,
                tiers=utils.get_pydantic_model(
                    tiers, Optional[List[models.components.DtoCreatePriceTier]]
                ),
                transform_quantity=utils.get_pydantic_model(
                    transform_quantity,
                    Optional[models.components.PriceTransformQuantity],
                ),
            ),
        )

        req = self._build_request_async(
            method="PUT",
            path="/prices/{id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.body,
                False,
                False,
                "json",
                models.components.DtoUpdatePriceRequest,
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="updatePrice",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.components.DtoPriceResponse, http_res)
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.FlexpriceDefaultError("Unexpected response received", http_res)

    def delete_price(
        self,
        *,
        id: str,
        end_date: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.components.DtoSuccessResponse:
        r"""Delete price

        Use when retiring a price (e.g. end-of-life or replacement). Optional effective date or cascade for subscriptions.

        :param id: Price ID
        :param end_date:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.operations.DeletePriceRequest(
            id=id,
            body=models.components.DtoDeletePriceRequest(
                end_date=end_date,
            ),
        )

        req = self._build_request(
            method="DELETE",
            path="/prices/{id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.body,
                False,
                False,
                "json",
                models.components.DtoDeletePriceRequest,
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="deletePrice",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.components.DtoSuccessResponse, http_res
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.FlexpriceDefaultError("Unexpected response received", http_res)

    async def delete_price_async(
        self,
        *,
        id: str,
        end_date: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.components.DtoSuccessResponse:
        r"""Delete price

        Use when retiring a price (e.g. end-of-life or replacement). Optional effective date or cascade for subscriptions.

        :param id: Price ID
        :param end_date:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.operations.DeletePriceRequest(
            id=id,
            body=models.components.DtoDeletePriceRequest(
                end_date=end_date,
            ),
        )

        req = self._build_request_async(
            method="DELETE",
            path="/prices/{id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.body,
                False,
                False,
                "json",
                models.components.DtoDeletePriceRequest,
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="deletePrice",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.components.DtoSuccessResponse, http_res
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.FlexpriceDefaultError("Unexpected response received", http_res)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .status import Status
from flexprice_py.types import BaseModel, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Dict, Optional
from typing_extensions import NotRequired, TypedDict


class DtoSubscriptionPhaseResponseTypedDict(TypedDict):
    created_at: NotRequired[str]
    created_by: NotRequired[str]
    end_date: NotRequired[str]
    r"""EndDate is when the phase ends (nil if phase is still active or indefinite)"""
    environment_id: NotRequired[str]
    r"""EnvironmentID is the environment identifier for the phase"""
    id: NotRequired[str]
    r"""ID is the unique identifier for the subscription phase"""
    metadata: NotRequired[Dict[str, str]]
    start_date: NotRequired[str]
    r"""StartDate is when the phase starts"""
    status: NotRequired[Status]
    subscription_id: NotRequired[str]
    r"""SubscriptionID is the identifier for the subscription"""
    tenant_id: NotRequired[str]
    updated_at: NotRequired[str]
    updated_by: NotRequired[str]


class DtoSubscriptionPhaseResponse(BaseModel):
    created_at: Optional[str] = None

    created_by: Optional[str] = None

    end_date: Optional[str] = None
    r"""EndDate is when the phase ends (nil if phase is still active or indefinite)"""

    environment_id: Optional[str] = None
    r"""EnvironmentID is the environment identifier for the phase"""

    id: Optional[str] = None
    r"""ID is the unique identifier for the subscription phase"""

    metadata: Optional[Dict[str, str]] = None

    start_date: Optional[str] = None
    r"""StartDate is when the phase starts"""

    status: Optional[Status] = None

    subscription_id: Optional[str] = None
    r"""SubscriptionID is the identifier for the subscription"""

    tenant_id: Optional[str] = None

    updated_at: Optional[str] = None

    updated_by: Optional[str] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "created_at",
                "created_by",
                "end_date",
                "environment_id",
                "id",
                "metadata",
                "start_date",
                "status",
                "subscription_id",
                "tenant_id",
                "updated_at",
                "updated_by",
            ]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .basesdk import BaseSDK
from .httpclient import AsyncHttpClient, ClientOwner, HttpClient, close_clients
from .sdkconfiguration import SDKConfiguration
from .utils.logger import Logger, get_default_logger
from .utils.retries import RetryConfig
from flexprice_py import models
from flexprice_py._hooks import SDKHooks
from flexprice_py.types import OptionalNullable, UNSET
import httpx
import importlib
import sys
from typing import Any, Callable, Optional, TYPE_CHECKING, Union, cast
import weakref

if TYPE_CHECKING:
    from flexprice_py.addons import Addons
    from flexprice_py.alerts import Alerts
    from flexprice_py.costs import Costs
    from flexprice_py.coupons import Coupons
    from flexprice_py.credit_grants import CreditGrants
    from flexprice_py.credit_notes import CreditNotes
    from flexprice_py.customers import Customers
    from flexprice_py.entitlements import Entitlements
    from flexprice_py.entity_integration_mappings import EntityIntegrationMappings
    from flexprice_py.environments import Environments
    from flexprice_py.events import Events
    from flexprice_py.features import Features
    from flexprice_py.groups import Groups
    from flexprice_py.integrations import Integrations
    from flexprice_py.invoices import Invoices
    from flexprice_py.payments import Payments
    from flexprice_py.plans import Plans
    from flexprice_py.price_units import PriceUnits
    from flexprice_py.prices import Prices
    from flexprice_py.rbac import Rbac
    from flexprice_py.scheduled_tasks import ScheduledTasks
    from flexprice_py.secrets import Secrets
    from flexprice_py.subscriptions import Subscriptions
    from flexprice_py.tasks import Tasks
    from flexprice_py.tax_associations import TaxAssociations
    from flexprice_py.tax_rates import TaxRates
    from flexprice_py.tenants import Tenants
    from flexprice_py.users import Users
    from flexprice_py.wallets import Wallets
    from flexprice_py.webhooks import Webhooks
    from flexprice_py.workflows import Workflows


class Flexprice(BaseSDK):
    r"""Flexprice API: Flexprice API provides billing, metering, and subscription management for SaaS and usage-based products. Use it to manage customers, plans, invoices, payments, usage events, and entitlements. Authenticate with an API key in the x-api-key header."""

    addons: "Addons"
    entitlements: "Entitlements"
    alerts: "Alerts"
    costs: "Costs"
    coupons: "Coupons"
    credit_grants: "CreditGrants"
    credit_notes: "CreditNotes"
    customers: "Customers"
    wallets: "Wallets"
    invoices: "Invoices"
    entity_integration_mappings: "EntityIntegrationMappings"
    environments: "Environments"
    events: "Events"
    features: "Features"
    groups: "Groups"
    payments: "Payments"
    plans: "Plans"
    prices: "Prices"
    price_units: "PriceUnits"
    rbac: "Rbac"
    secrets: "Secrets"
    integrations: "Integrations"
    subscriptions: "Subscriptions"
    tasks: "Tasks"
    scheduled_tasks: "ScheduledTasks"
    tax_associations: "TaxAssociations"
    tax_rates: "TaxRates"
    tenants: "Tenants"
    users: "Users"
    webhooks: "Webhooks"
    workflows: "Workflows"
    _sub_sdk_map = {
        "addons": ("flexprice_py.addons", "Addons"),
        "entitlements": ("flexprice_py.entitlements", "Entitlements"),
        "alerts": ("flexprice_py.alerts", "Alerts"),
        "costs": ("flexprice_py.costs", "Costs"),
        "coupons": ("flexprice_py.coupons", "Coupons"),
        "credit_grants": ("flexprice_py.credit_grants", "CreditGrants"),
        "credit_notes": ("flexprice_py.credit_notes", "CreditNotes"),
        "customers": ("flexprice_py.customers", "Customers"),
        "wallets": ("flexprice_py.wallets", "Wallets"),
        "invoices": ("flexprice_py.invoices", "Invoices"),
        "entity_integration_mappings": (
            "flexprice_py.entity_integration_mappings",
            "EntityIntegrationMappings",
        ),
        "environments": ("flexprice_py.environments", "Environments"),
        "events": ("flexprice_py.events", "Events"),
        "features": ("flexprice_py.features", "Features"),
        "groups": ("flexprice_py.groups", "Groups"),
        "payments": ("flexprice_py.payments", "Payments"),
        "plans": ("flexprice_py.plans", "Plans"),
        "prices": ("flexprice_py.prices", "Prices"),
        "price_units": ("flexprice_py.price_units", "PriceUnits"),
        "rbac": ("flexprice_py.rbac", "Rbac"),
        "secrets": ("flexprice_py.secrets", "Secrets"),
        "integrations": ("flexprice_py.integrations", "Integrations"),
        "subscriptions": ("flexprice_py.subscriptions", "Subscriptions"),
        "tasks": ("flexprice_py.tasks", "Tasks"),
        "scheduled_tasks": ("flexprice_py.scheduled_tasks", "ScheduledTasks"),
        "tax_associations": ("flexprice_py.tax_associations", "TaxAssociations"),
        "tax_rates": ("flexprice_py.tax_rates", "TaxRates"),
        "tenants": ("flexprice_py.tenants", "Tenants"),
        "users": ("flexprice_py.users", "Users"),
        "webhooks": ("flexprice_py.webhooks", "Webhooks"),
        "workflows": ("flexprice_py.workflows", "Workflows"),
    }

    def __init__(
        self,
        server_url: str,
        api_key_auth: Union[str, Callable[[], str]],
        client: Optional[HttpClient] = None,
        async_client: Optional[AsyncHttpClient] = None,
        retry_config: OptionalNullable[RetryConfig] = UNSET,
        timeout_ms: Optional[int] = None,
        debug_logger: Optional[Logger] = None,
    ) -> None:
        r"""Instantiates the SDK configuring it with the provided parameters.

        :param api_key_auth: The api_key_auth required for authentication
        :param server_idx: The index of the server to use for all methods
        :param server_url: The server URL to use for all methods
        :param url_params: Parameters to optionally template the server URL with
        :param client: The HTTP client to use for all synchronous methods
        :param async_client: The Async HTTP client to use for all asynchronous methods
        :param retry_config: The retry configuration to use for all supported methods
        :param timeout_ms: Optional request timeout applied to each operation in milliseconds
        """
        client_supplied = True
        if client is None:
            client = httpx.Client(follow_redirects=True)
            client_supplied = False

        assert issubclass(
            type(client), HttpClient
        ), "The provided client must implement the HttpClient protocol."

        async_client_supplied = True
        if async_client is None:
            async_client = httpx.AsyncClient(follow_redirects=True)
            async_client_supplied = False

        if debug_logger is None:
            debug_logger = get_default_logger()

        assert issubclass(
            type(async_client), AsyncHttpClient
        ), "The provided async_client must implement the AsyncHttpClient protocol."

        security: Any = None
        if callable(api_key_auth):
            # pylint: disable=unnecessary-lambda-assignment
            security = lambda: models.components.Security(api_key_auth=api_key_auth())
        else:
            security = models.components.Security(api_key_auth=api_key_auth)

        BaseSDK.__init__(
            self,
            SDKConfiguration(
                client=client,
                client_supplied=client_supplied,
                async_client=async_client,
                async_client_supplied=async_client_supplied,
                security=security,
                server_url=server_url,
                retry_config=retry_config,
                timeout_ms=timeout_ms,
                debug_logger=debug_logger,
            ),
            parent_ref=self,
        )

        hooks = SDKHooks()

        # pylint: disable=protected-access
        self.sdk_configuration.__dict__["_hooks"] = hooks

        self.sdk_configuration = hooks.sdk_init(self.sdk_configuration)

        weakref.finalize(
            self,
            close_clients,
            cast(ClientOwner, self.sdk_configuration),
            self.sdk_configuration.client,
            self.sdk_configuration.client_supplied,
            self.sdk_configuration.async_client,
            self.sdk_configuration.async_client_supplied,
        )

    def dynamic_import(self, modname, retries=3):
        for attempt in range(retries):
            try:
                return importlib.import_module(modname)
            except KeyError:
                # Clear any half-initialized module and retry
                sys.modules.pop(modname, None)
                if attempt == retries - 1:
                    break
        raise KeyError(f"Failed to import module '{modname}' after {retries} attempts")

    def __getattr__(self, name: str):
        if name in self._sub_sdk_map:
            module_path, class_name = self._sub_sdk_map[name]
            try:
                module = self.dynamic_import(module_path)
                klass = getattr(module, class_name)
                instance = klass(self.sdk_configuration, parent_ref=self)
                setattr(self, name, instance)
                return instance
            except ImportError as e:
                raise AttributeError(
                    f"Failed to import module {module_path} for attribute {name}: {e}"
                ) from e
            except AttributeError as e:
                raise AttributeError(
                    f"Failed to find class {class_name} in module {module_path} for attribute {name}: {e}"
                ) from e

        raise AttributeError(
            f"'{type(self).__name__}' object has no attribute '{name}'"
        )

    def __dir__(self):
        default_attrs = list(super().__dir__())
        lazy_attrs = list(self._sub_sdk_map.keys())
        return sorted(list(set(default_attrs + lazy_attrs)))

    def __enter__(self):
        return self

    async def __aenter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        if (
            self.sdk_configuration.client is not None
            and not self.sdk_configuration.client_supplied
        ):
            self.sdk_configuration.client.close()
        self.sdk_configuration.client = None

    async def __aexit__(self, exc_type, exc_val, exc_tb):
        if (
            self.sdk_configuration.async_client is not None
            and not self.sdk_configuration.async_client_supplied
        ):
            await self.sdk_configuration.async_client.aclose()
        self.sdk_configuration.async_client = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from flexprice_py.types import BaseModel, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class GithubComFlexpriceFlexpriceInternalTypesValueTypedDict(TypedDict):
    array: NotRequired[List[str]]
    boolean: NotRequired[bool]
    date_: NotRequired[str]
    number: NotRequired[float]
    string: NotRequired[str]


class GithubComFlexpriceFlexpriceInternalTypesValue(BaseModel):
    array: Optional[List[str]] = None

    boolean: Optional[bool] = None

    date_: Annotated[Optional[str], pydantic.Field(alias="date")] = None

    number: Optional[float] = None

    string: Optional[str] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["array", "boolean", "date", "number", "string"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


try:
    GithubComFlexpriceFlexpriceInternalTypesValue.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .dto_walletresponse import DtoWalletResponse, DtoWalletResponseTypedDict
from .dto_wallettransactionresponse import (
    DtoWalletTransactionResponse,
    DtoWalletTransactionResponseTypedDict,
)
from flexprice_py.types import BaseModel, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class DtoTopUpWalletResponseTypedDict(TypedDict):
    invoice_id: NotRequired[str]
    r"""Invoice ID if an invoice was created (only for PURCHASED_CREDIT_INVOICED)"""
    wallet: NotRequired[DtoWalletResponseTypedDict]
    wallet_transaction: NotRequired[DtoWalletTransactionResponseTypedDict]


class DtoTopUpWalletResponse(BaseModel):
    invoice_id: Optional[str] = None
    r"""Invoice ID if an invoice was created (only for PURCHASED_CREDIT_INVOICED)"""

    wallet: Optional[DtoWalletResponse] = None

    wallet_transaction: Optional[DtoWalletTransactionResponse] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["invoice_id", "wallet", "wallet_transaction"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .basesdk import BaseSDK
from flexprice_py import errors, models, utils
from flexprice_py._hooks import HookContext
from flexprice_py.types import OptionalNullable, UNSET
from flexprice_py.utils.unmarshal_json_response import unmarshal_json_response
from typing import Any, Dict, List, Mapping, Optional, Union


class PriceUnits(BaseSDK):
    def list_price_units(
        self,
        *,
        status: Optional[str] = None,
        limit: Optional[int] = 50,
        offset: Optional[int] = 0,
        sort: Optional[str] = None,
        order: Optional[models.operations.ListPriceUnitsOrder] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.components.DtoListPriceUnitsResponse:
        r"""List price units

        Use when listing price units (e.g. in a catalog or when creating prices). Returns a paginated list; supports status, sort, and pagination.

        :param status: Filter by status
        :param limit: Limit number of results
        :param offset: Offset for pagination
        :param sort: Sort field
        :param order: Sort order
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.operations.ListPriceUnitsRequest(
            status=status,
            limit=limit,
            offset=offset,
            sort=sort,
            order=order,
        )

        req = self._build_request(
            method="GET",
            path="/prices/units",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="listPriceUnits",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.components.DtoListPriceUnitsResponse, http_res
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.FlexpriceDefaultError("Unexpected response received", http_res)

    async def list_price_units_async(
        self,
        *,
        status: Optional[str] = None,
        limit: Optional[int] = 50,
        offset: Optional[int] = 0,
        sort: Optional[str] = None,
        order: Optional[models.operations.ListPriceUnitsOrder] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.components.DtoListPriceUnitsResponse:
        r"""List price units

        Use when listing price units (e.g. in a catalog or when creating prices). Returns a paginated list; supports status, sort, and pagination.

        :param status: Filter by status
        :param limit: Limit number of results
        :param offset: Offset for pagination
        :param sort: Sort field
        :param order: Sort order
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.operations.ListPriceUnitsRequest(
            status=status,
            limit=limit,
            offset=offset,
            sort=sort,
            order=order,
        )

        req = self._build_request_async(
            method="GET",
            path="/prices/units",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="listPriceUnits",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.components.DtoListPriceUnitsResponse, http_res
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.FlexpriceDefaultError("Unexpected response received", http_res)

    def create_price_unit(
        self,
        *,
        base_currency: str,
        code: str,
        conversion_rate: str,
        name: str,
        symbol: str,
        metadata: Optional[Dict[str, str]] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.components.DtoCreatePriceUnitResponse:
        r"""Create price unit

        Use when defining a new unit of measure for pricing (e.g. GB, API call, seat). Ideal for metered or usage-based prices.

        :param base_currency: base_currency  is the currency that the price unit is based on
        :param code:
        :param conversion_rate: ConversionRate defines the exchange rate from this price unit to the base currency.
            This rate is used to convert amounts in the custom price unit to the base currency for storage and billing.

            Conversion formula:
            price_unit_amount * conversion_rate = base_currency_amount

            Example:
            If conversion_rate = \"0.01\" and base_currency = \"usd\":
            100 price_unit tokens * 0.01 = 1.00 USD

            Note: Rounding precision is determined by the base currency (e.g., USD uses 2 decimal places, JPY uses 0).
        :param name:
        :param symbol:
        :param metadata:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.components.DtoCreatePriceUnitRequest(
            base_currency=base_currency,
            code=code,
            conversion_rate=conversion_rate,
            metadata=metadata,
            name=name,
            symbol=symbol,
        )

        req = self._build_request(
            method="POST",
            path="/prices/units",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request,
                False,
                False,
                "json",
                models.components.DtoCreatePriceUnitRequest,
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="createPriceUnit",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "201", "application/json"):
            return unmarshal_json_response(
                models.components.DtoCreatePriceUnitResponse, http_res
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.FlexpriceDefaultError("Unexpected response received", http_res)

    async def create_price_unit_async(
        self,
        *,
        base_currency: str,
        code: str,
        conversion_rate: str,
        name: str,
        symbol: str,
        metadata: Optional[Dict[str, str]] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.components.DtoCreatePriceUnitResponse:
        r"""Create price unit

        Use when defining a new unit of measure for pricing (e.g. GB, API call, seat). Ideal for metered or usage-based prices.

        :param base_currency: base_currency  is the currency that the price unit is based on
        :param code:
        :param conversion_rate: ConversionRate defines the exchange rate from this price unit to the base currency.
            This rate is used to convert amounts in the custom price unit to the base currency for storage and billing.

            Conversion formula:
            price_unit_amount * conversion_rate = base_currency_amount

            Example:
            If conversion_rate = \"0.01\" and base_currency = \"usd\":
            100 price_unit tokens * 0.01 = 1.00 USD

            Note: Rounding precision is determined by the base currency (e.g., USD uses 2 decimal places, JPY uses 0).
        :param name:
        :param symbol:
        :param metadata:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.components.DtoCreatePriceUnitRequest(
            base_currency=base_currency,
            code=code,
            conversion_rate=conversion_rate,
            metadata=metadata,
            name=name,
            symbol=symbol,
        )

        req = self._build_request_async(
            method="POST",
            path="/prices/units",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request,
                False,
                False,
                "json",
                models.components.DtoCreatePriceUnitRequest,
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="createPriceUnit",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "201", "application/json"):
            return unmarshal_json_response(
                models.components.DtoCreatePriceUnitResponse, http_res
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.FlexpriceDefaultError("Unexpected response received", http_res)

    def get_price_unit_by_code(
        self,
        *,
        code: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.components.DtoPriceUnitResponse:
        r"""Get price unit by code

        Use when resolving a price unit by code (e.g. from an external catalog or config). Ideal for integrations.

        :param code: Price unit code
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.operations.GetPriceUnitByCodeRequest(
            code=code,
        )

        req = self._build_request(
            method="GET",
            path="/prices/units/code/{code}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="getPriceUnitByCode",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "404", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.components.DtoPriceUnitResponse, http_res
            )
        if utils.match_response(http_res, ["400", "404"], "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.FlexpriceDefaultError("Unexpected response received", http_res)

    async def get_price_unit_by_code_async(
        self,
        *,
        code: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.components.DtoPriceUnitResponse:
        r"""Get price unit by code

        Use when resolving a price unit by code (e.g. from an external catalog or config). Ideal for integrations.

        :param code: Price unit code
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.operations.GetPriceUnitByCodeRequest(
            code=code,
        )

        req = self._build_request_async(
            method="GET",
            path="/prices/units/code/{code}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="getPriceUnitByCode",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "404", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.components.DtoPriceUnitResponse, http_res
            )
        if utils.match_response(http_res, ["400", "404"], "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.FlexpriceDefaultError("Unexpected response received", http_res)

    def query_price_unit(
        self,
        *,
        end_time: Optional[str] = None,
        expand: Optional[str] = None,
        filters: Optional[
            Union[
                List[models.components.FilterCondition],
                List[models.components.FilterConditionTypedDict],
            ]
        ] = None,
        limit: Optional[int] = None,
        offset: Optional[int] = None,
        order: Optional[models.components.PriceUnitFilterOrder] = None,
        price_unit_ids: Optional[List[str]] = None,
        sort: Optional[
            Union[
                List[models.components.SortCondition],
                List[models.components.SortConditionTypedDict],
            ]
        ] = None,
        start_time: Optional[str] = None,
        status: Optional[models.components.Status] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.components.DtoListPriceUnitsResponse:
        r"""Query price units

        Use when searching or listing price units (e.g. admin catalog). Returns a paginated list; supports filtering and sorting.

        :param end_time:
        :param expand:
        :param filters: filters allows complex filtering based on multiple fields
        :param limit:
        :param offset:
        :param order:
        :param price_unit_ids:
        :param sort:
        :param start_time:
        :param status:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.components.PriceUnitFilter(
            end_time=end_time,
            expand=expand,
            filters=utils.get_pydantic_model(
                filters, Optional[List[models.components.FilterCondition]]
            ),
            limit=limit,
            offset=offset,
            order=order,
            price_unit_ids=price_unit_ids,
            sort=utils.get_pydantic_model(
                sort, Optional[List[models.components.SortCondition]]
            ),
            start_time=start_time,
            status=status,
        )

        req = self._build_request(
            method="POST",
            path="/prices/units/search",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request, False, False, "json", models.components.PriceUnitFilter
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="queryPriceUnit",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.components.DtoListPriceUnitsResponse, http_res
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.FlexpriceDefaultError("Unexpected response received", http_res)

    async def query_price_unit_async(
        self,
        *,
        end_time: Optional[str] = None,
        expand: Optional[str] = None,
        filters: Optional[
            Union[
                List[models.components.FilterCondition],
                List[models.components.FilterConditionTypedDict],
            ]
        ] = None,
        limit: Optional[int] = None,
        offset: Optional[int] = None,
        order: Optional[models.components.PriceUnitFilterOrder] = None,
        price_unit_ids: Optional[List[str]] = None,
        sort: Optional[
            Union[
                List[models.components.SortCondition],
                List[models.components.SortConditionTypedDict],
            ]
        ] = None,
        start_time: Optional[str] = None,
        status: Optional[models.components.Status] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.components.DtoListPriceUnitsResponse:
        r"""Query price units

        Use when searching or listing price units (e.g. admin catalog). Returns a paginated list; supports filtering and sorting.

        :param end_time:
        :param expand:
        :param filters: filters allows complex filtering based on multiple fields
        :param limit:
        :param offset:
        :param order:
        :param price_unit_ids:
        :param sort:
        :param start_time:
        :param status:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.components.PriceUnitFilter(
            end_time=end_time,
            expand=expand,
            filters=utils.get_pydantic_model(
                filters, Optional[List[models.components.FilterCondition]]
            ),
            limit=limit,
            offset=offset,
            order=order,
            price_unit_ids=price_unit_ids,
            sort=utils.get_pydantic_model(
                sort, Optional[List[models.components.SortCondition]]
            ),
            start_time=start_time,
            status=status,
        )

        req = self._build_request_async(
            method="POST",
            path="/prices/units/search",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request, False, False, "json", models.components.PriceUnitFilter
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="queryPriceUnit",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.components.DtoListPriceUnitsResponse, http_res
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.FlexpriceDefaultError("Unexpected response received", http_res)

    def get_price_unit(
        self,
        *,
        id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.components.DtoPriceUnitResponse:
        r"""Get price unit

        Use when you need to load a single price unit (e.g. for display or when creating a price).

        :param id: Price unit ID
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.operations.GetPriceUnitRequest(
            id=id,
        )

        req = self._build_request(
            method="GET",
            path="/prices/units/{id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="getPriceUnit",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "404", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.components.DtoPriceUnitResponse, http_res
            )
        if utils.match_response(http_res, ["400", "404"], "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.FlexpriceDefaultError("Unexpected response received", http_res)

    async def get_price_unit_async(
        self,
        *,
        id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.components.DtoPriceUnitResponse:
        r"""Get price unit

        Use when you need to load a single price unit (e.g. for display or when creating a price).

        :param id: Price unit ID
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.operations.GetPriceUnitRequest(
            id=id,
        )

        req = self._build_request_async(
            method="GET",
            path="/prices/units/{id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="getPriceUnit",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "404", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.components.DtoPriceUnitResponse, http_res
            )
        if utils.match_response(http_res, ["400", "404"], "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.FlexpriceDefaultError("Unexpected response received", http_res)

    def update_price_unit(
        self,
        *,
        id: str,
        metadata: Optional[Dict[str, str]] = None,
        name: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.components.DtoPriceUnitResponse:
        r"""Update price unit

        Use when renaming or updating metadata for a price unit. Code is immutable once created.

        :param id: Price unit ID
        :param metadata:
        :param name:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.operations.UpdatePriceUnitRequest(
            id=id,
            body=models.components.DtoUpdatePriceUnitRequest(
                metadata=metadata,
                name=name,
            ),
        )

        req = self._build_request(
            method="PUT",
            path="/prices/units/{id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.body,
                False,
                False,
                "json",
                models.components.DtoUpdatePriceUnitRequest,
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="updatePriceUnit",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "404", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.components.DtoPriceUnitResponse, http_res
            )
        if utils.match_response(http_res, ["400", "404"], "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.FlexpriceDefaultError("Unexpected response received", http_res)

    async def update_price_unit_async(
        self,
        *,
        id: str,
        metadata: Optional[Dict[str, str]] = None,
        name: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.components.DtoPriceUnitResponse:
        r"""Update price unit

        Use when renaming or updating metadata for a price unit. Code is immutable once created.

        :param id: Price unit ID
        :param metadata:
        :param name:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.operations.UpdatePriceUnitRequest(
            id=id,
            body=models.components.DtoUpdatePriceUnitRequest(
                metadata=metadata,
                name=name,
            ),
        )

        req = self._build_request_async(
            method="PUT",
            path="/prices/units/{id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.body,
                False,
                False,
                "json",
                models.components.DtoUpdatePriceUnitRequest,
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="updatePriceUnit",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "404", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.components.DtoPriceUnitResponse, http_res
            )
        if utils.match_response(http_res, ["400", "404"], "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.FlexpriceDefaultError("Unexpected response received", http_res)

    def delete_price_unit(
        self,
        *,
        id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.components.DtoSuccessResponse:
        r"""Delete price unit

        Use when removing a price unit that is no longer needed. Fails if any price references this unit.

        :param id: Price unit ID
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.operations.DeletePriceUnitRequest(
            id=id,
        )

        req = self._build_request(
            method="DELETE",
            path="/prices/units/{id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="deletePriceUnit",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "404", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.components.DtoSuccessResponse, http_res
            )
        if utils.match_response(http_res, ["400", "404"], "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.FlexpriceDefaultError("Unexpected response received", http_res)

    async def delete_price_unit_async(
        self,
        *,
        id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.components.DtoSuccessResponse:
        r"""Delete price unit

        Use when removing a price unit that is no longer needed. Fails if any price references this unit.

        :param id: Price unit ID
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.operations.DeletePriceUnitRequest(
            id=id,
        )

        req = self._build_request_async(
            method="DELETE",
            path="/prices/units/{id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="deletePriceUnit",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "404", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.components.DtoSuccessResponse, http_res
            )
        if utils.match_response(http_res, ["400", "404"], "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.FlexpriceDefaultError("Unexpected response received", http_res)
